/* This file is part of KDevelop

    SPDX-FileCopyrightText: 2015 Kevin Funk <kfunk@kde.org>

    SPDX-License-Identifier: LGPL-2.0-only
*/

#ifndef PHPDOCSDEBUG_H
#define PHPDOCSDEBUG_H

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(DOCS)

#endif /* PHPDOCSDEBUG_H */
