/* This file is part of KDevelop

    SPDX-FileCopyrightText: 2014 Milian Wolff <mail@milianw.de>

    SPDX-License-Identifier: LGPL-2.0-only
*/

#ifndef PARSERDEBUG_H
#define PARSERDEBUG_H

#include <QLoggingCategory>
Q_DECLARE_LOGGING_CATEGORY(PARSER)

#endif /* PARSERDEBUG_H */
