package apackage;

class Base {
    public Base() {}
    Type method(Type type) { return type; }
	public void methodWithVector(String[] args) {}
	public Type[] getSubs() { return null; }
	public final void finalMethod() {}
	String[] stringVector;
}

