#include "comments-class-methods.h"
#include "comments-class-members.h"
#include "comments-class-enums.h"
#include "comments-class-operators.h"
#include "comments-class-typedefs.h"
#include "comments-file-variant1.h"
#include "comments-file-variant2.h"
#include "comments-file-variant3.h"
#include "comments-class.h"
