
// class comments_class_enum_test
class comments_class_enum_test {
public:
    // enum test c++ style single line comment
    enum test1 { A, B, C};

    /*
     * enum test2
     * c-style multi line comment
     */
    enum test2 { A, B, C};

    /* enum test3 c-style single lime comment */
    enum test3 { A, B, C};
};
