class ExplicitConstructorDeclaration {
    explicit ExplicitConstructorDeclaration(QString &param);
};

class ExplicitConstructorDefinition {
    explicit ExplicitConstructorDefinition(QString &param) {}
};
