/*
    SPDX-License-Identifier: GPL-2.0-or-later

    SPDX-FileCopyrightText: 2020 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
*/

#ifndef _N_STEREOATTRS__H
#define _N_STEREOATTRS__H

// Maximum number of tagged values per stereotype
#define N_STEREOATTRS 3

#endif
