
// class comments_class_typedef_test
class comments_class_typedef_test {
public:
    // enim test c++ style single line comment
    typedef int type1;

    /*
     * typedef test2
     * c-style multi line comment
     */
    typedef int type2;

    /* typedef test3 c-style single lime comment */
    typedef int type3;
};