/* This file is part of KDevelop

    SPDX-FileCopyrightText: 2014 Milian Wolff <mail@milianw.de>

    SPDX-License-Identifier: LGPL-2.0-only
*/

#ifndef PHPDEBUG_H
#define PHPDEBUG_H

#include <QLoggingCategory>
Q_DECLARE_LOGGING_CATEGORY(PHP)

#endif /* PHPDEBUG_H */
