class DestructorDeclaration {
    DestructorDeclaration(QString &param);
    ~DestructorDeclaration(QString &param);
};

class DestructorDefinition {
    DestructorDefinition(QString &param) {}
    ~DestructorDefinition(QString &param) {}
};
