/*
    SPDX-License-Identifier: GPL-2.0-or-later
    SPDX-FileCopyrightText: 2004-2020 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
*/

#ifndef CODEACCESSORMETHODLIST_H
#define CODEACCESSORMETHODLIST_H

#include <QList>

// forward declarations
class CodeAccessorMethod;

typedef QList<CodeAccessorMethod*> CodeAccessorMethodList;
typedef QListIterator<CodeAccessorMethod*> CodeAccessorMethodListIt;

#endif
