/* This file is part of KDevelop

    SPDX-FileCopyrightText: 2015 Kevin Funk <kfunk@kde.org>

    SPDX-License-Identifier: LGPL-2.0-only
*/

#include "phpdocsdebug.h"

Q_LOGGING_CATEGORY(DOCS, "kdevelop.languages.php.docs")
