/*

    SPDX-License-Identifier: GPL-2.0-or-later
    SPDX-FileCopyrightText: 2004-2007 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
*/

#ifndef UMLTEMPLATELIST_H
#define UMLTEMPLATELIST_H

#include <qlist.h>

// forward declaration
class UMLTemplate;

typedef QList<UMLTemplate*> UMLTemplateList;
typedef QListIterator<UMLTemplate*> UMLTemplateListIt;

#endif
