/*
    SPDX-License-Identifier: GPL-2.0-or-later
    SPDX-FileCopyrightText: 2004-2007 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
*/

#ifndef UMLENUMLITERALLIST_H
#define UMLENUMLITERALLIST_H

#include <QList>

// forward declaration
class UMLEnumLiteral;

typedef QList<UMLEnumLiteral*> UMLEnumLiteralList;
typedef QListIterator<UMLEnumLiteral*> UMLEnumLiteralListIt;

#endif
