/* This file is part of KDevelop

    SPDX-FileCopyrightText: 2014 Milian Wolff <mail@milianw.de>

    SPDX-License-Identifier: LGPL-2.0-only
*/

#include "phpdebug.h"

Q_LOGGING_CATEGORY(PHP, "kdevelop.languages.php")
