/*
    SPDX-License-Identifier: GPL-2.0-or-later
    SPDX-FileCopyrightText: 2003-2012 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
*/

#ifndef UMLSTEREOTYPELIST_H
#define UMLSTEREOTYPELIST_H

#include <QList>

// forward declaration
class UMLStereotype;

typedef QList<UMLStereotype*> UMLStereotypeList;
typedef QListIterator<UMLStereotype*> UMLStereotypeListIt;

#endif
