/*
    SPDX-License-Identifier: GPL-2.0-or-later
    SPDX-FileCopyrightText: 2020-2020 Umbrello UML Modeller Authors <umbrello-devel@kde.org>
*/

#ifndef FILE_UTILS_H
#define FILE_UTILS_H

#include <QString>

namespace File_Utils {
    QString xmlCatalogFilePath();
}

#endif // FILE_UTILS_H
