# coding=UTF-8

class Visibility():
  __private=0
  _protected=0
  public=0

  # public by default
  def __init__(self):
    pass

  def public(self):
    pass

  def _protected(self):
    pass

  def __private(self):
    pass




