// independent comment 1

// class comments_class_test1a c++ style single line comment
class comments_class_test1a {};

// independent comment 2

/// class comments_class_test1b doxygen c++ style single line comment
class comments_class_test1b {};

// independent comment 3

// class comments_class_test1c c++ style multi line comment 1 of 2 lines
// class comments_class_test1c c++ style multi line comment 2 of 2 lines
class comments_class_test1c {};

// independent comment 4

// class comments_class_test1d c++ style multi line comment 1 of 3 lines
// class comments_class_test1d c++ style multi line comment 2 of 3 lines
// class comments_class_test1d c++ style multi line comment 3 of 3 lines
class comments_class_test1d {};

// independent comment 5

/// class comments_class_test1e doxygen c++ style multi line comment 1 of 3 lines
/// class comments_class_test1e doxygen c++ style multi line comment 2 of 3 lines
/// class comments_class_test1e doxygen c++ style multi line comment 3 of 3 lines
class comments_class_test1e {};

// independent comment 6

/*
 * class comments_class_test2a
 * c-style multiline comment
 */
class comments_class_test2a {};

// independent comment 7

/**
 * class comments_class_test2b
 * doxygen c-style multiline comment
 */
class comments_class_test2b {};

/* class comments_class_test2c c-style single line comment */
class comments_class_test2c {};

